/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttComm.c
 *
 * Date:        March 1995
 *
 * Purpose:     Simple UT-SNMP commands interface
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <unistd.h>
#include <strings.h>
#include <arpa/inet.h>
#include <UT-SNMP.h>
#include <ttComm.h>

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrNetMonitor
 *
 * Arguments:	type, the arrived management communication type (OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS MngrNetMonitor( AsnInt *type )
{
  return( utbLook( type ) );
}



/*
 * -----------------------------------------------------------------------
 *
 * Function:	MakeManagerConn
 *
 * Arguments:	name, Name of the Connection (IN)
 *
 * Return:	OK or ERROR
 *
 * Purpose:	Open manager Connection
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS MakeManagerConn( AsnOcts *name )
{  
  return utbOpen( name, UT_MANAGER_ROLE );
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:	CloseConnection
 *
 * Arguments:	none
 *
 * Return:	OK or ERROR
 *
 * Purpose:	Close Connection
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
  STATUS
CloseConnection()
{ 
  return ( utbClose() != OK );
}




/*
 * -----------------------------------------------------------------------
 *
 * Function:	MakeAddr
 *
 * Arguments:	taddr, internet address (out)
 *		ipaddr, IP-address (ascii) (in)
 *
 * Return:	STATUS, OK.
 *
 * Purpose:	This function convertes an IP-address into an 
 *		internet-address
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
STATUS MakeAddr(  AsnOcts **taddr, AsnOcts *ipaddr )
{
  AsnInt inetaddr;

  inetaddr = inet_addr( ipaddr->octs ) ;
  *taddr = mOcNew(6) ;
  memcpy( (*taddr)->octs, &inetaddr, (*taddr)->octetLen = sizeof( inetaddr ) ) ;

  return OK;
}                

/*
 * -----------------------------------------------------------------------
 *
 * Function:	GetAddr
 *
 * Arguments:	ipaddr, IP-address (ascii) (out)
 *		taddr, internet address (in)
 *
 * Return:	STATUS, OK.
 *
 * Purpose:	This function convertes an internet-address into an 
 *		IP-address
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
STATUS GetAddr(  AsnOcts **ipaddr, AsnOcts *taddr )
{
  char *inetntoa;

  inetntoa = inet_ntoa( taddr->octs ) ;
  *ipaddr = mOcNew(15);
  memcpy( (*ipaddr)->octs, inetntoa, (*ipaddr)->octetLen = strlen( inetntoa ) ) ;

  return OK;
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:	MakeAsnOcts
 *
 * Arguments:	asnocts, pointer to var (out)
 *		name,  name of the text that must be placed in asnocts
 *
 * Return:	STATUS, OK.
 *
 * Purpose:	sinple creation of AsnOcts vars
 *
 * Remarks:		
 *
 * -----------------------------------------------------------------------
 */
STATUS MakeAsnOcts(AsnOcts **asnocts, char *name)
{
  *asnocts = mOcNew( strlen(name) );
  strcpy((*asnocts)->octs, name);
  (*asnocts)->octetLen = strlen(name);
 
  return OK;
}




static char *snmpstatuserrors[] = {
    "noError",
    "tooBig",
    "noSuchName",
    "badValue",
    "readOnly",
    "genErr",
    "noAccess",
    "wrongType",
    "wrongLength",
    "wrongEncoding",
    "wrongValue",
    "noCreation",
    "inconsistentValue",
    "resourceUnavailable",
    "commitFailed",
    "undoFailed",
    "authorizationError",
    "notWritable",
    "inconsistentName",
    "unknown",
    "providError",
    "timeout",
    "unknownContextLocalEntity",
    "noUniqueDestination",
    "noSuchContextLocal",
    "noUndoError",
    "commitFailed",
    "unknownHost",
    "noSuchQos",
    "mibRegFailed",
    "mibUnregFailed",
    "informRegFailed",
    "informUnregFailed",
    "trapRegFailed",
    "trapUnregFailed",
    "illegalOperation"
};


/*
 * -----------------------------------------------------------------------
 *
 * Function:	SnmpStatusError
 *
 * Arguments:	errnr, number of the snmp status error (IN)
 *
 * Return:	error string
 *
 * Purpose:	give error string of errnr
 *
 * Remarks:		
 *
 * -----------------------------------------------------------------------
 */
char *SnmpStatusError( AsnInt errnr )
{
  if ((errnr >= 0) && (errnr <= 35))  
    return snmpstatuserrors[errnr];
  else return snmpstatuserrors[UT_UNKNOWN];
}


static char *snmpexeptions[] = {
  "noExeption",
  "noSuchObject",
  "noSuchInstance",
  "endOfMibView"
};
/*
 * -----------------------------------------------------------------------
 *
 * Function:	SnmpExeption
 *
 * Arguments:	expnr, number of the snmp exeption (IN)
 *
 * Return:	exeption string
 *
 * Purpose:	give exeption string of expnr
 *
 * Remarks:		
 *
 * -----------------------------------------------------------------------
 */
char *SnmpExeption( AsnInt expnr )
{
  if ((expnr >= 0) && (expnr <= 3))  
    return snmpexeptions[expnr];
  else return NULL;
}
