/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttDebug.c
 *
 * Date:        March 1995
 *
 * Purpose:     snmp command inplementation
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttAgentList.h>
#include <ttCallList.h>
#include <ttTimer.h>
#include <ttDebug.h>


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<snmp> list
 *
 * TCL Args:	agent: list of known agents
 *		call : list of waiting calls
 *
 * TCL example:	snmp list call
 *
 * Return:	list or error
 *
 * Purpose:	check what the system knows
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int ListCommand( ClientData clientdata, 
                 Tcl_Interp *interp, int argc, char *argv[] ) 
{
  if (argc < 3) {
    Tcl_SetResult( interp, "syntax: snmp list [agent / call]", TCL_VOLATILE );
    return TCL_ERROR;
  }

  switch (*argv[2]) {
    case 'a':            
      if (strcmp(argv[2], "agent") == 0) {   /* listsnmp agent */
        PrintAgentList();
        break;
      }
    case 'c':
      if (strcmp(argv[2], "call") == 0) {   /* listsnmp call */
        PrintCallList();
        break;
      }
  }

  return TCL_OK;
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	help
 *
 * TCL Args:	none
 *
 * TCL example:	<snmp> help
 *
 * Return:	prints list of commands
 *
 * Purpose:	give help
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int HelpSnmpCommand( ClientData clientdata, 
                     Tcl_Interp *interp, int argc, char *argv[] ) 
{
    fprintf(stderr,"<x> : user defined name\n");
    fprintf(stderr,"?x? : options\n");
    fprintf(stderr,"\n");
    fprintf(stderr,"newsnmp    <agent> <ip-addr>\n");
    fprintf(stderr,"<agent>    get     <callback>  <vBinds>\n");
    fprintf(stderr,"<agent>    next    <callback>  <vBinds>\n");
    fprintf(stderr,"<agent>    bulk    <callback>  <vBinds>\n");
    fprintf(stderr,"<agent>    set     <callback>  <vBinds>\n");
    fprintf(stderr,"<agent>    wait    call-number\n");
    fprintf(stderr,"<agent>    config  ?retries / timeout / set-value?\n");
    fprintf(stderr,"snmp       ?help / list / timer?\n");
    fprintf(stderr,"snmp list  ?agent / call?\n");
    fprintf(stderr,"snmp timer ?newtime?\n");
  
  return TCL_OK;
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	snmp
 *
 * TCL Args:	/ help / list / timer
 *
 * TCL example:	snmp help
 *
 * Return:	depends on option
 *
 * Purpose:	
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int SnmpCommand( ClientData clientdata, 
                 Tcl_Interp *interp, int argc, char *argv[] ) 
{
  if (argc < 2) {
    HelpSnmpCommand( clientdata, interp, argc--, argv++ );
    return TCL_OK;
  }

  switch (*argv[1]) {
    case 'h':            
      if (strcmp(argv[1], "help") == 0) {   /* help */
        HelpSnmpCommand( clientdata, interp, argc, argv );
        break;
      }
    case 'l':
      if (strcmp(argv[1], "list") == 0) {   /* list */
        ListCommand( clientdata, interp, argc, argv );
        break;
      }
    case 't':
      if (strcmp(argv[1], "timer") == 0) {   /* timer */
        TimerCommand( clientdata, interp, argc, argv );
        break;
      }
     default:
        Tcl_SetResult( interp, "syntax: snmp ?help / list / timer?", TCL_VOLATILE );
        return TCL_ERROR;
   }

  return TCL_OK;
}

