/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttFoid.c
 *
 * Date:        March 1995
 *
 * Purpose:     Friendly name tcl interface
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttComm.h>
#include <ut-foid.h>

FoidT	*pFoidDataBase;
int	FoidInstalled = 0;


STATUS Foid_Init() 
{
  STATUS result;

  result = foidOpen( &pFoidDataBase );
  if (result == OK)
    FoidInstalled = 1;
  return (result);
}


STATUS Foid_Exit()
{
  FoidInstalled = 0;
  return (foidClose( &pFoidDataBase ) );
}


STATUS FoidToOID(AsnOcts *Friendly, AsnOcts **OID )
{
  int nDescription;
  int nError;
  AsnOcts *dummy = mOcNew( 255 );

  if (FoidInstalled == 0) 
    return FAILED;

  return (foidConv( pFoidDataBase, Friendly->octs, *dummy, **OID, 
                    &nDescription, &nError ) );
}


STATUS OIDToFoid(AsnOcts *OID, AsnOcts **Friendly )
{
  int nDescription;
  int nError;
  AsnOcts *dummy = mOcNew( 255 );

  if (FoidInstalled == 0) 
    return FAILED;
  
  return (foidConv( pFoidDataBase, OID->octs, **Friendly, *dummy,  
                    &nDescription, &nError ) );
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	
 *
 * TCL Args:	
 *
 * TCL example:	
 *
 * Return:	
 *
 * Purpose:	
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int FoidToOIDCommand( ClientData clientdata, 
                      Tcl_Interp *interp, int argc, char *argv[] ) 
{
  STATUS result;
  AsnOcts *Friendly;
  AsnOcts *OID = mOcNew( 255 );

  if (argc < 2) {
    Tcl_SetResult( interp, "expecting one argument", TCL_VOLATILE );
    return TCL_ERROR;
  }

  if (FoidInstalled == 0) {
    Tcl_SetResult( interp, "foid database not installed", TCL_VOLATILE );
    return TCL_ERROR;
  }


  MakeAsnOcts( &Friendly, argv[1] );

  result = FoidToOID( Friendly, &OID );
  switch (result) {
    case OK:
          Tcl_SetResult( interp, OID->octs, TCL_VOLATILE );
          return TCL_OK;
    case FAILED:
          Tcl_SetResult( interp, "0", TCL_VOLATILE );
          return TCL_OK;
    case ERROR:
          Tcl_SetResult( interp, "Not enough memory", TCL_VOLATILE );
          return TCL_ERROR;
     default:
          Tcl_SetResult( interp, "Unexpected error", TCL_VOLATILE );
          return TCL_ERROR;
  }      
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	
 *
 * TCL Args:	
 *
 * TCL example:	
 *
 * Return:	
 *
 * Purpose:	
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int OIDToFoidCommand( ClientData clientdata, 
                      Tcl_Interp *interp, int argc, char *argv[] ) 
{
  STATUS result;
  AsnOcts *Friendly = mOcNew( 255 );
  AsnOcts *OID;

  if (argc < 2) {
    Tcl_SetResult( interp, "expecting one argument", TCL_VOLATILE );
    return TCL_ERROR;
  }

  if (FoidInstalled == 0) {
    Tcl_SetResult( interp, "foid database not installed", TCL_VOLATILE );
    return TCL_ERROR;
  }


  MakeAsnOcts( &OID, argv[1] );

  result = OIDToFoid(OID, &Friendly);
  switch (result) {
    case OK:
          Tcl_SetResult( interp, Friendly->octs, TCL_VOLATILE );
          return TCL_OK;
    case FAILED:
          Tcl_SetResult( interp, "0", TCL_VOLATILE );
          return TCL_OK;
    case ERROR:
          Tcl_SetResult( interp, "Not enough memory", TCL_VOLATILE );
          return TCL_ERROR;
     default:
          Tcl_SetResult( interp, "Unexpected error", TCL_VOLATILE );
          return TCL_ERROR;
  }      
}

