/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttGet.c
 *
 * Date:        March 1995
 *
 * Purpose:     Get command implmentation
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttsnmp.h>
#include <ttComm.h>
#include <ttAgentList.h>
#include <ttCallList.h>
#include <ttCallID.h>
#include <ttVarBind.h>
#include <ttGet.h>


/*
 * -----------------------------------------------------------------------
 *
 * Function:	SendGet
 *
 * Arguments:	interp, TCL interpreter pointer (IN)
 *		taddr, internet address (IN)
 *		CallID, ID number of the call (IN)
 *		OIDList, oidlist where the get must operate on (IN)
 *
 * Return:	OK or ERROR	
 *
 * Purpose:	perform a get-command on the agent: taddr
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS SendGet( Tcl_Interp *interp,
                AsnOcts *taddr, 
                AsnInt CallID, 
                char *OIDList )
{
  AsnVbl  *vbl = mVlNew( );       /* var bind list */
  AsnOcts *le  = mOcNew( 64 ); 
  AsnOid  *lt  = mOiNew( 64 ); 
 
    /* create localtime: lt */  
  memcpy( lt->octs, UT_CURRENTTIME.octs, 
          lt->octetLen = UT_CURRENTTIME.octetLen ) ;
  le->octetLen = 0;	/* local entity = empty */
 
    /* add OID-list to the var bind list vbl */
  AddReadVarList( interp, OIDList, vbl );

  return( utbGetRq( CallID, taddr, UT_NONE_QOS, le, lt, vbl )  );
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<agent> get
 *
 * TCL Args:	<callback> <vBind>
 *
 * TCL example:	<agent> get <callback> 1.3.6.1.2.1.1.1.0
 *
 * Return:	CallID
 *
 * Purpose:	Handle get command for an agent
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int SnmpGetCommand( ClientData clientdata, 
                    Tcl_Interp *interp, int argc, char *argv[] ) 
{  
  AsnOcts    *agentname;
  TAgentInfo  agentinfo;
  TCallInfo   callinfo;
  AsnInt      CallID;

  MakeAsnOcts( &agentname, argv[0] ); /* create AsnOcts type of procname */

  if (GetAgent(agentname, &agentinfo ) == OK) {		/* agent proc known ? */

      /* add call to CallList */  
    CallID = NextCallID();
    strcpy( callinfo.agentName, argv[0] );     /* agent proc name */
    strcpy( callinfo.tclcallback, argv[2] );   /* callback procedure name */
    callinfo.timeout = agentinfo.timeout;      /* set timeout value */
    AddCall(CallID, callinfo );

      /* do get command */
    if (SendGet( interp, &agentinfo.agentaddr, CallID, argv[3] ) == OK) 
    { 
       /* give call id back to user */
      sprintf( interp->result, "%i", CallID );	 
      return TCL_OK;
    }
    else Tcl_SetResult( interp, "Error in sending get command", TCL_VOLATILE );

  }
  else Tcl_SetResult( interp, "Agent proc unknown", TCL_VOLATILE );


  return TCL_ERROR;
}

