/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttNewSnmp.c
 *
 * Date:        March 1995
 *
 * Purpose:     NewSnmp command implementation
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttComm.h>
#include <ttAgentList.h>
#include <ttWait.h>
#include <ttGet.h>
#include <ttNext.h> 
#include <ttBulk.h>
#include <ttSet.h>
#include <ttNewSnmp.h>


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<agent>
 *
 * TCL Args:    [get / next / bulk / set / config]
 *
 * TCL example: 
 *
 * Return:      
 *
 * Purpose:     basic context function handler
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */


/*
 * Handle agent commands
 *
 * command	status
 * config	no
 * operation
 *   get 	yes
 *   next 	yes
 *   set 	yes
 *   bulk	yes
 * wait		yes
 * register	no
 * destroy	no
 *
 */
int SnmpHandleCommand( ClientData clientdata, 
                       Tcl_Interp *interp, int argc, char *argv[] ) 
{
  int result = OK;

  if (argc < 2) {
    Tcl_SetResult( interp, "expecting at least 1 argument", TCL_VOLATILE );
    return TCL_ERROR;
  }

  switch (*argv[1]) {
    case 'g': 
        if (strcmp(argv[1], "get") == 0) {
          result = SnmpGetCommand( clientdata, interp, argc, argv );
          break;
        }
    case 'n': 
        if (strcmp(argv[1], "next") == 0) {
          result = SnmpNextCommand( clientdata, interp, argc, argv );
          break;
        }
    case 'b': 
        if (strcmp(argv[1], "bulk") == 0) {
          result = SnmpBulkCommand( clientdata, interp, argc, argv );
          break;
        }
    case 's': 
        if (strcmp(argv[1], "set") == 0) {
          result = SnmpSetCommand( clientdata, interp, argc, argv );
          break;
        }
    case 'c':
        if (strcmp(argv[1], "config") == 0) {
          result = SnmpAgentConfig(clientdata, interp, argc, argv );
          break;
        }
    case 'w':
        if (strcmp(argv[1], "wait") == 0) {
          result = SnmpWaitCommand(clientdata, interp, argc, argv );
          break;
        }
    default:
        fprintf(stderr,"unknown command\n");
        break;
  }

  return result;   
}

/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	newsnmp
 *
 * TCL Args:	newsnmp <agent proc name> <ipaddress>
 *
 * TCL example:	newsnmp agent 130.89.16.108
 *
 * Return:      none
 *
 * Purpose:     create new agent context
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
int NewSnmpCommand( ClientData clientdata, 
                    Tcl_Interp *interp, int argc, char *argv[] ) 
{
  AsnOcts     *AgentName, *AgentAddr, *taddr; 
  TAgentInfo  AgentInfo;

  if (argc < 2) {
    Tcl_SetResult( interp, "expecting 2 arguments", TCL_VOLATILE );
    return TCL_ERROR;
  }

  MakeAsnOcts( &AgentName, argv[1] );  /* get agent proc name */
  MakeAsnOcts( &AgentAddr, argv[2] );  /* get agent ip address */
  MakeAddr( &taddr, AgentAddr );
 
    /* add agent information in AgentList */
  AgentInfo.agentaddr = *taddr;
  AgentInfo.retries   = 0;
  AgentInfo.timeout   = 10;
  AgentInfo.maxrept   = 1;
  AgentInfo.nonrptr   = 0;
  AddAgent( AgentName, AgentInfo );  

    /* create new command to talk to agent */
  Tcl_CreateCommand( interp, argv[1], SnmpHandleCommand, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );

    /* give agent proc back */
  Tcl_SetResult( interp, argv[1], TCL_VOLATILE );
  return TCL_OK;
}
