/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttSet.c
 *
 * Date:        March 1995
 *
 * Purpose:     Set command implmentaion
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttsnmp.h>
#include <ttAgentList.h>
#include <ttCallList.h>
#include <ttCallID.h>
#include <ttVarBind.h>
#include <ttSet.h>


/*
 * -----------------------------------------------------------------------
 *
 * Function:	SendSet
 *
 * Arguments:	taddr, internet address (IN)
 *		OID,   oid where the get must operate on
 *		Syntax, Syntax description
 *		Value, Value to be set in the oid
 *
 * Return:	OK or FAILED	
 *
 * Purpose:	perform a get-command on the agent: taddr
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */


STATUS SendSet( Tcl_Interp *interp,
                AsnOcts    *taddr, 
                AsnInt     CallID, 
                char       *SetList )
{
  AsnVbl  *vbl = mVlNew( );       /* var bind list */
  AsnOcts *le  = mOcNew( 64 ); 
  AsnOid  *lt  = mOiNew( 64 ); 
  STATUS s;
 
    /* create localtime: lt */  
  memcpy( lt->octs, UT_CURRENTTIME.octs, 
          lt->octetLen = UT_CURRENTTIME.octetLen ) ;
  le->octetLen = 0;	/* local entity = empty */
 
    /* add OID, Syntax en Value to the var bind list vbl */
  if (AddWriteVarList( interp, SetList, vbl ) == OK) 
    return( utbSetRq( CallID, taddr, UT_NONE_QOS, le, lt, vbl ) );
  return FAILED;
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<agent> set
 *
 * TCL Args:	<callback> <vBind>
 *
 * TCL example:	<agent> set <callback> {1.3.6.1.2.1.1.1.0 DisplayString "aiko" }
 *
 * Return:	CallID
 *
 * Purpose:	Handle get command for an agent
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int SnmpSetCommand( ClientData clientdata, 
                    Tcl_Interp *interp, int argc, char *argv[] ) 
{  
  AsnOcts    *agentname;
  TAgentInfo  agentinfo;
  TCallInfo   callinfo;
  AsnInt      CallID;

  if (argc < 4) { 
    Tcl_SetResult( interp, "Expecting 4 arguments", TCL_VOLATILE );
    return TCL_ERROR;
  } 

  MakeAsnOcts( &agentname, argv[0] ); /* create AsnOcts type of procname */

  if (GetAgent(agentname, &agentinfo ) != OK) {		/* agent proc known ? */
    Tcl_SetResult( interp, "Agent proc unknown", TCL_VOLATILE );
    return TCL_ERROR;
  }

    /* add call to CallList */  
  CallID = NextCallID();
  strcpy( callinfo.agentName, argv[0] );     /* agent proc name */
  strcpy( callinfo.tclcallback, argv[2] );   /* callback procedure name */
  callinfo.timeout = agentinfo.timeout;      /* set timeout value */
  AddCall(CallID, callinfo );

    /* do set command */
  if (SendSet( interp, &agentinfo.agentaddr, CallID, argv[3] ) == OK) { 
    sprintf( interp->result, "%i", CallID );  /* give call id back to user */
    return TCL_OK;
  }
  else {
    Tcl_SetResult( interp, "Error in sending set command", TCL_VOLATILE );
    return TCL_ERROR;
  }
}

