/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttTimer.c
 *
 * Date:        March 1995
 *
 * Purpose:     Timers functions
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/


#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttComm.h>
#include <ttCallList.h>
#include <ttCallBack.h>
#include <ttTimer.h>

int TimerPollTime = 2000;  /* poll every 2 seconds the ut-snmp retrieve stack */
int TimerOutTime  = 1000;  /* every second time out checks */


/*
 * -----------------------------------------------------------------------
 *
 * Function:    TimerPollProc
 *
 * Arguments:   clientdata
 *
 * Return:      none
 *
 * Purpose:     Check if there is any callback from an agent
 *
 * Remarks:	This function slows down the performans of the system!
 *
 * -----------------------------------------------------------------------
 */
void TimerPollProc( ClientData clientdata ) 
{
  AsnInt callbacktype = 0;

    /* Is there any UT-SNMP callback ? */
  if ( MngrNetMonitor( &callbacktype ) == OK ) 
  {
     HandleCallBack( callbacktype );
  }

  SetPollTimer();  /* set a new timer */
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:    SetPollTimer
 *
 * Arguments:   none
 *
 * Return:      none
 *
 * Purpose:     Set a new timer period for proc TimerProc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void SetPollTimer()
{
  Tk_CreateTimerHandler( TimerPollTime, TimerPollProc, (ClientData *) NULL );
}


void TimeOutCheckProc( int CallID, TCallInfo *callinfo )
{
  callinfo->timeout--;

  if (callinfo->timeout == 0) {                /* timeout ? */
    Retrieve( CallID, 0, UT_TIMEOUT, NULL );   /* notify user */       
    return;
  }
}

/*
 * -----------------------------------------------------------------------
 *
 * Function:    TimerOutProc
 *
 * Arguments:   clientdata
 *
 * Return:      none
 *
 * Purpose:     
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void TimerOutProc( ClientData clientdata ) 
{
  ForEachCall( &TimeOutCheckProc );

  SetOutTimer();  /* set a new timer */
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:    SetOutTimer
 *
 * Arguments:   none
 *
 * Return:      none
 *
 * Purpose:     
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void SetOutTimer()
{
  Tk_CreateTimerHandler( TimerOutTime, TimerOutProc, 
                         (ClientData *) NULL );
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<snmp> timer
 *
 * TCL Args:	 	: give the current timer value
 *		[n]	: set a new timer value	
 *
 * TCL example:	snmp timer 500
 *
 * Return:	none
 *
 * Purpose:	change the polling timer
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
int TimerCommand( ClientData clientdata, 
                  Tcl_Interp *interp, int argc, char *argv[] )
{
  char result[255];
   
  if (argc == 2) {                             /* get timer value ? */
    sprintf( result, "%i", TimerPollTime );
    Tcl_AppendResult( interp, &result[0], (char *) NULL );
  }  
  else                                         /* set timer value */
  {
    Tcl_AppendResult( interp, argv[2], (char *) NULL );
    TimerPollTime = atoi( argv[2] );
  }
  
  return TCL_OK;
}
